//////////////////////////////////////////////////////////////////////////
// JJ Foote w/ parts by Michael Thornbury                               //
// PlaySPC_GTK                                                          //
// Version 0.2.7 - February 21, 2003                                    //
//                                                                      //
// This program is free software; you can redistribute it and/or modify //
// it under the terms of the GNU General Public License as published by //
// the Free Software Foundation; either version 2 of the License, or    //
// (at your option) any later version.                                  //
//                                                                      //
// This program is distributed in the hope that it will be useful,      //
// but WITHOUT ANY WARRANTY; without even the implied warranty of       //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        //
// GNU General Public License for more details.                         //
//////////////////////////////////////////////////////////////////////////

#ifndef __PLAYSPC_GTK_H
#define __PLAYSPC_GTK_H

// Compile-time constants - change to fit your system
#define MAX_STRING_SIZE	200
#define MAX_PLAYLIST_ENTRIES 500

// internal socket queue
#define QLEN 32

// program queue length, a multiple of QLEN
#define QUEUE_LENGTH QLEN

// Includes
#include <gtk/gtk.h>
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <libgen.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include "shared.h"

// XPM Images
#include "add.xpm"
#include "remove.xpm"
#include "moveup.xpm"
#include "movedown.xpm"
#include "play.xpm"
#include "pause.xpm"
#include "stop.xpm"
#include "fastforward.xpm"
#include "rewind.xpm"

extern int passiveTCP(int, int);

// Structure for id666 tags in SPC files
struct id666 {
    char songtitle[50];
    char gametitle[50];
    char dumpername[25];
    char comments[50];
    char dumpdate[25];
    int songlength_raw;
    char songlength[10];
    int fadeout; // This field is currently ignored by PlaySPC_GTK
    char songauthor[50];
    char dumpprog[15];
};

/* Load default preferences, etc. from .rc file */
gboolean load_rc();

/* Save preferences, etc. to .rc file */
gboolean save_rc();

/* Parses an SPC file's ID666 header tag (if it has one), and stores the results in the struct passed in */
gboolean read_id666_tag(char* spcfile, struct id666* tag);

/* Write a modified ID666 tag back to an SPC file.  Note that the change
 * will not be permanent if the SPC file is inside a RAR archive.
 */
gboolean write_id666_tag(char* spcfile, struct id666* tag);

// A standard means of passing in a filename, a "prettier" version of it,
// and a place to put the finished list name.
void fill_name(char*, int);

/* Extracts all SPC files in a RAR archive to a temporary directory, and populates the playlist */
gboolean rar_extract(char* rarfile);

/* Some basic initialization */
int init();

/* Encoder initialization */
int init2();

void sigpipe_handler(int);

/* Handles input from the input plugin. */
void* input_plugin_handler();

void* input_encoder_handler();

void* fifo_output_handler();

void* shoutcast_server();

void* shoutcast_client();

int handle(int, char*);

/* Stops SPC playback */
void stop();

// Handles making sure everything comes out at a sane speed.
void alarm_handler(int);

/* Starts playing the current SPC file in the playlist */
int play();

/* If using ID666 timers, PlaySPC_GTK sets a GTK timeout for the length of the
 * song, and will call this function when the time is up.  This function will
 * then stop playback, and automatically move on to the next song in the playlist
 */
int next_track();

/* Generic GTK callback function that handles most of the buttons in the various
 * windows.  A text string is assigned to each button, and passed here as 'data'
 */
void buttonhandler(GtkWidget* widget, gpointer data);

/* GTK callback function for the dialog box brought up by 'File/Open' */
gboolean file_selection_handler(GtkWidget* widget, GtkFileSelection* fs);
gboolean file_selection_handler_add(GtkWidget* widget, GtkFileSelection* fs);
gboolean file_selection_handler_joined(GtkWidget* widget, GtkFileSelection* fs);

/* The actual file loader */
gboolean open_multi_file_container(char* filename);

/* The actual playlist loaders, both version. */
void load_playlist_v1();
void load_playlist_v2();

/* Takes name from file, then returns the playable full name in file. */
int load_from_rar(char* name, char* file);

/* GTK callback function for the dialog box brought up by 'File/Save As' */
void file_save_handler(GtkWidget* widget, GtkFileSelection* fs);

/* GTK callback function for the dialog box brought up by the 'ADD' toolbar button */
void file_add_handler(GtkWidget* widget, GtkFileSelection* fs);

/* GTK callback functions for changing any information in the 'Options/View File Info' dialog */
void cb_text_entry(GtkWidget* widget, gpointer data);
void cb_id666_songlength(GtkWidget* widget, gpointer data);

/* GTK signal handler for selecting an item from the playlist */
void clist_sel(GtkWidget* clist, gint row, gint col, GdkEventButton* event, gpointer data);

/* GTK signal handler for selecting an item from the playlist */
void clist_unsel(GtkWidget* clist, gint row, gint col, GdkEventButton* event, gpointer data);

/* GTK signal handler for closing a window (besides the main window) */
gint destroy(GtkWidget* widget, GdkEvent* event, gpointer data);

/* GTK callback function for the 'File/New' menu item */
void cb_new(GtkWidget* widget, gpointer data);

/* GTK callback function for the 'File/Open' menu item */
void cb_open(GtkWidget* widget, gpointer data);

/* GTK callback function for the 'File/Save' and 'File/Save As' menu items */
void cb_save(gpointer data, guint action, GtkWidget* widget);

/* Generic message box */
void message_box(char* title, char* text);

/* The actual save routines */

void save_rar(void);
/* The thread that does the actual work. */
void* saving_rar();

/* This doesn't take enough time to warrent a thread. */
void save_playlist(void);

/* GTK callback functino for the 'ADD' toolbar button */
void cb_add(GtkWidget* widget, gpointer data);

/* GTK callback function for the 'Options/Preferences' menu item */
void cb_prefs(GtkWidget* widget, gpointer data);

/* Rebuilds the playlist to concur with the new preferences. */
void rebuild_clist();

/* GTK callback function for the 'Options/View File Info' menu item */
void cb_view_info(GtkWidget* widget, gpointer data);

/* Generates a spinbutton/label combo and displays them, returning the spinbox. */
GtkWidget* gtk_spinbutton_new_with_label(GtkWidget* table, char* text,
                                         gfloat value, gfloat lower, gfloat upper, gfloat step_increment, gfloat page_increment,
                                         gfloat climb_rate);

/* GTK callback function for the 'Help/About' menu item */
void cb_about(GtkWidget* widget, gpointer data);

/* Clean up properly when we quit the application */
void clean_up_and_quit(int);

/* Tell the input plug-in to exit. */
void input_quit();

/* Part of the ItemFactory approach to building a menubar */
void get_main_menu(GtkWidget* window, GtkWidget** menubar);

#endif
